import sys
import argparse
from .common import *


def get_argparser_linear_kmin(parser):
    parser.add_argument("-s",
                        "--start",
                        help="starting length (default: -s 10)",
                        action="store",
                        nargs='?',
                        default=10,
                        type=int)
    parser.add_argument("target_fn",
                        help="Filename of the reference sequence file or directory.",
                        nargs='*')



#                             -*- Mode: Python -*-
# Graph.py --- Generic graph class (directed, weighted)
#

import logging as log
import numpy as np

class Graph:
    """Nodes are linked with weighted edges dictating the shortest
    paths from each edge towards start and end nodes.

    Nodes are identified by their index: ``0 .. (n-1)``.

    Attributes
    ---------
    n : int
        Number of nodes (N).
    w : (N,N) array-like
        Graph with N nodes.
    edge_set : set
        A set of all relevant edges.
    first_node : int
        First node index.
    last_node : int
        Last node index.
    before : (N,) array-like
        The value at a specific position inside the list represents
        the node ID *preceding* the node whose ID is the current
        position.
    after : (N,) array-like
        The value at a specific position inside the list represents
        the node ID *following* the node whose ID is the current
        position.

    Methods
    -------
    init_paths
    all_shortest
    """

    def __init__(self, n):
        self.n = n
        self.w = np.empty((n, n), dtype=np.float32)
        self.w.fill(np.inf)
        self.edge_set = set()

        self.first_node = 0
        self.last_node = 0
        self.before = None
        self.after = None

        #self.ref_path = []

    def __getitem__(self, indices):
        (i, j) = indices
        return self.w[i, j]

    def __setitem__(self, indices, new_val):
        (i, j) = indices
        self.w[i, j] = new_val
        self.edge_set.add((i, j))

    def _get_paths(self, start, w):
        """Follow path from start until exhaustion of all kmers.

        Arguments
        ---------
        start : int
            ID of initial node to start with.
        w : numpy.ndarray
            A 2-dimensional weighted array containing weight
            values for each pair of edges.

        Returns
        -------
        numpy.ndarray
            List of preceding nodes as dictated by the weights
        """

        # a list of kmers (indices) and their corresponding
        # predecessors (values)
        prev = np.empty(self.n, dtype=np.int32)
        prev.fill(-1)

        # dist will contain the cumulative weights of forward
        # kmers as they appear during kmer walking
        dist = np.empty(self.n, dtype=np.float32)
        dist.fill(np.inf)
        unvisited = np.ones(self.n, dtype=bool)

        def visit(i):
            # ndist is the weights for all kmers forward of `i` +
            # the current cumulative weight at the stage of kmer `i`
            ndist = w[i, :] + dist[i]

            # only keep forward kmers which:
            #   - were not selected before (np.inf in dist)
            #   - are currently in a reference edge even if they
            #     were already found in a variant edge, their
            #     value in dist will be overwritten by the lower
            #     weight and their previous kmer will be the current
            #     kmer `i`.
            ind = tuple([ndist < dist])
            dist[ind] = ndist[ind]
            prev[ind] = i

        # ensure we start with the start kmer, at this point dist
        # is filled with +np.inf except for `start` which is 0
        dist[start] = 0
        while any(unvisited):
            # select passed over kmer with smallest weight from
            # previous iterations
            unv_ix = np.where(unvisited)[0]
            i = unv_ix[dist[unv_ix].argmin()]
            # find and annotate forward kmers for kmer `i`
            visit(i)
            unvisited[i] = False

        return prev

    def init_paths(self, first_node, last_node):
        """For each node, find its preceding (array `before`) and
        following (array `after`) node based on the weights in
        the graph.

        Construct a one-to-one mapping of nodes from each extremity
        of the reference, and get rid of reference edges (discovered
        by following the smallest weights from start to end).

        Follow the path of least-resistence / smallest weight
        starting from node `start` and continuously affect each
        node with its deduced predecessor.

        Follow paths from both sides of the reference to be able
        to construct the shortest paths from each side of an
        alternative edge.

        Here is a schematic representation of how the path is
        constructed. When 2 alternative branches exist, branched
        nodes are affected with the same preceding node. However,
        when the branching has to close, only one predecessor can
        be chosen, and it will always be the one with the lowest
        weight.

        | ``|------------------------------------------------------------------|``
        | ``|               • <- • <- •                                        |``
        | ``|             /             \   <= lower weight == is kept         |``
        | ``| • <- • <- •                 • <- • <- •                          |``
        | ``|             \            (/)  <= higher weight == is overwritten |``
        | ``|               • <- • <- •                                        |``
        | ``|------------------------------------------------------------------|``

        Notes
        -----
        In extremely rare situations, if the reference path (small
        weights) goes on for a very long time, the cumulative
        weights could end up catching up to the alternative path and
        become higher weights, which would cause an inversion
        of reference and alternative paths. An example would be if
        a reference path with a weight of 0.01 is 100 times longer
        than an alternative path with weights of 1, we may then get
        to a point where the cumulative weight of the reference
        is higher than that of the alternative.

        Arguments
        ---------
            first_node : int
                First node ID.
            last_node : int
                Last node ID.
        """

        self.first_node = first_node
        self.last_node = last_node
        self.before = self._get_paths(self.first_node, self.w)
        self.after = self._get_paths(self.last_node, self.w.transpose())

        # Load up and remove edges from the ref path. We know for a fact
        # that starting from the first node in list `after` will guarantee
        # that all edges will come from the reference (i.e., following the
        # path of least resistance). We would also get to the same result
        # by starting with the last node in list `before`.
        #path = [self.first_node]
        removed = 0
        curs = set(np.where(self.before == self.first_node)[0])
        for cur in curs:
            #path = [self.first_node]
            last_cur = None  # ensures we keep only one edge from the reference
            while self.after[cur] != -1:
                cur = self.after[cur]
                #path.append(cur)
                if last_cur and (last_cur, cur) in self.edge_set:  # we might have taken care of this edge already
                    self.edge_set.remove((last_cur, cur))
                    log.debug("Removing (%d, %d)", last_cur, cur)
                    removed += 1
                last_cur = cur
            #self.ref_path.append(path)
        log.info("Removed %d ref edges.", removed)

    def _get_shortest(self, a, b):
        """Return shortest path passing through edge (a, b)"""

        def follow(start, prev):
            cur = start
            while prev[cur] != -1:
                cur = prev[cur]
                path.append(cur)

        path = [b, a]
        follow(a, self.before)
        path.reverse()
        follow(b, self.after)

        ## Only keep paths from source to sink
        if path[0] != self.first_node or path[-1] != self.last_node:
            path = None

        return path

    def all_shortest(self):
        """Find every unique path that can be constructed from a
        non-reference edge by following the paths in `before` and
        `after`.

        Returns
        -------
        list
            List of unique shortest paths walked from all edges
        """

        log.info("%d edges in non-ref edge set.", len(self.edge_set))

        all_paths = set()
        for (i, j) in self.edge_set:
            log.debug("Computing shortest path through edge: (%d, %d)", i, j)
            path = self._get_shortest(i, j)
            if path:
                all_paths.add(tuple(path))

        return list(all_paths)



from .common import *


def get_argparser_find_mut(parser):
    parser.add_argument(
        "-c", "--count",
        help="Minimum occurence needed for exploration of alternative (default: -c 5)",
        action="store",
        nargs='?',
        default=5,
        type=int)
    parser.add_argument(
        "-p", "--ratio",
        help="Minimum occurence ratio needed for exploration of alternative (default: -p 0.05)",
        action="store",
        nargs='?',
        default=0.05,
        type=float)
    parser.add_argument(
        "-s", "--steps",
        help="Maximum steps to discover a new branch on a target sequence (default: -s 500)",
        action="store",
        nargs='?',
        default=500,
        type=int)
    parser.add_argument(
        "-b", "--branchs",
        help="Maximum branchs until getback to target sequence (default: -b 10)",
        action="store",
        nargs='?',
        default=10,
        type=int)
    parser.add_argument(
        "-n", "--nodes",
        help="Maximum nodes/kmers queried from jellyfish database (default: -n 5000)",
        action="store",
        nargs='?',
        default=10000,
        type=int)
    parser.add_argument(
        "-g", "--graphical",
        help="Display coverage graph.",
        action="store_true")
    parser.add_argument(
        "-v", "--verbose",
        help="Get more information.",
        action="store_true")
    parser.add_argument(
        "-vv", "--debug",
        help="Get much more information.",
        action="store_true")
    parser.add_argument(
        "target_fn",
        help="Filename of the target sequence file or directory.",
        nargs='*')
    parser.add_argument(
        "jellyfish_fn",
        help="Filename of the jellyfish database.")



import argparse
import sys

from .argparser.find_mutation import get_argparser_find_mut
from .argparser.find_report import get_argparser_find_report
from .argparser.linear_kmin import get_argparser_linear_kmin
from .argparser.min_cov import get_argparser_min_cov

from .tools.find_mutation import main_find_mut
from .tools.find_report import main_find_report
from .tools.linear_kmin import main_linear_kmin
from .tools.min_cov import main_min_cov


# ###########################################################################
# Main function
def main():
    # sys.stderr.write("\n--------------------------------------------------------------\n")
    # sys.stderr.write("km.py: Tools for targeted variant detection.\n")
    # sys.stderr.write("This program was written by IRIC's bioinformatics platform\n")
    # sys.stderr.write("----------------------------------------------------------------\n\n")

    parser = argparse.ArgumentParser(prog='PROG')
    subparsers = parser.add_subparsers(help='sub-command help')

    # create the argparser for the "find_mutation" command
    find_mut = subparsers.add_parser(
        'find_mutation',
        help='Identify and quantify mutations from a target sequence and a k-mer database.'
    )
    find_mut.set_defaults(func=main_find_mut)
    get_argparser_find_mut(find_mut)

    # create the argparser for the "find_report" command
    find_report = subparsers.add_parser(
        'find_report',
        help='Parse find_mutation output and reformat it in a more user-friendly tabulated file.'
    )
    find_report.set_defaults(func=main_find_report)
    get_argparser_find_report(find_report)

    # create the argparser for the "linear_kmin" command
    linear_kmin = subparsers.add_parser(
        'linear_kmin',
        help='Find min k-length to decompose a target sequence in a linear graph.'
    )
    linear_kmin.set_defaults(func=main_linear_kmin)
    get_argparser_linear_kmin(linear_kmin)

    # create the argparser for the "min_cov" command
    min_cov = subparsers.add_parser(
        'min_cov',
        help='Compute coverage of target sequences.'
    )
    min_cov.set_defaults(func=main_min_cov)
    get_argparser_min_cov(min_cov)

    # display help if no arguments given
    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)

    # recover arguments
    args = parser.parse_args()

    # execute the command
    args.func(args, parser)



import sys
import argparse
from .common import *


def get_argparser_find_report(parser):
    parser._action_groups.pop()
    required = parser.add_argument_group('required arguments')
    optional = parser.add_argument_group('optional arguments')

    optional.add_argument(
        "-t",
        dest="target",
        help="Filename of the target sequence file",
        type=lambda x: is_valid_file(parser, x))

    required.add_argument(
        'infile',
        nargs='?',
        type=argparse.FileType('r'),
        default=sys.stdin)

    optional.add_argument(
        "-i",
        dest="info",
        help="Filter on info column (Default: vs_ref)",
        default="vs_ref",
        type=str)

    optional.add_argument(
        "-m",
        dest="min_cov",
        help="Min coverage allowed (Default: 1)",
        default=1,
        type=int)

    optional.add_argument(
        "-e", "--exclu",
        dest="exclu",
        help="Filename of a jf database, containing k-mers which can " +\
             "create false positive variants (as, a jf build on the transcriptome)",
        default="",
        type=str)

    optional.add_argument(
        "-f", "--format",
        dest="format",
        help="Option 'vcf': Output variants in VCF-like file format -- Option 'table':" +\
             "Group variants by position and return per-sample ratio",
        choices=['vcf', 'table'])



#                             -*- Mode: Python -*-
# Jellyfish.py --- Provides a python front-end to query a Jellyfish database.
#

import os
import sys
import json
import logging as log
try:
    import dna_jellyfish as jellyfish
except ModuleNotFoundError:
    import jellyfish

class Jellyfish:
    """A python interface for querying a Jellyfish database.

    Methods
    -------
    query
    get_child
    """

    def __init__(self, filename, cutoff=0.30, n_cutoff=500):
        self.jf = jellyfish.QueryMerFile(filename)
        self.k = jellyfish.MerDNA.k()
        self.filename = filename
        self.cutoff = cutoff
        self.n_cutoff = n_cutoff
        with open(filename, mode='rb') as f:
            header = f.readline().decode("ascii", errors='ignore')
        header = "{" + header.split("{", 1)[1]
        closed = -1
        h = "{"
        for x in header[1:]:
            if closed:
                h += x
                if x == "{":
                    closed -= 1
                elif x == "}":
                    closed += 1
            else:
                break
        header = h
        header = json.loads(header)
        self.canonical = header["canonical"]

    def query(self, seq):
        """Fetch kmer count data from database."""

        kmer = jellyfish.MerDNA(seq)
        if (self.canonical):
            kmer.canonicalize()
        return self.jf[kmer]

    def get_child(self, seq, forward=True):
        """Return all existing kmers that continue from input
        sequence.
        """

        child = []
        sum = 0
        for c in ['A', 'C', 'G', 'T']:
            if forward:
                c_seq = seq[1:] + c
            else:
                c_seq = c + seq[0:-1]
            c_count = self.query(c_seq)
            child.append((c_seq, c_count))
            sum += c_count
        threshold = max(sum * self.cutoff, self.n_cutoff)

        return [x[0] for x in [x for x in child if x[1] >= threshold]]



# find_mutation ---
#
#   Usage:  find_mutation <region_fasta or directory> <jellyfish_db>
import os
import sys
import time
import logging as log
from .. utils import MutationFinder as umf
from .. utils import common as uc
from .. utils import Sequence as us
from .. utils.Jellyfish import Jellyfish


# ###########################################################################
# Main function

def main_find_mut(args, argparser):
    time_start = time.time()

    if args.verbose:
        log.basicConfig(level=log.INFO, format="VERBOSE: %(message)s")

    if args.debug:
        log.basicConfig(level=log.DEBUG, format="VERBOSE: %(message)s")

    for k, v in vars(args).items():
        sys.stdout.write("#" + str(k) + ':' + str(v) + "\n")

    jf = Jellyfish(args.jellyfish_fn, cutoff=args.ratio, n_cutoff=args.count)

    seq_files = uc.target_2_seqfiles(args.target_fn)

    umf.MutationFinder.output_header()

    refpaths = []

    for seq_f in seq_files:

        (ref_name, ext) = os.path.splitext(os.path.basename(seq_f))

        ref_seqs, ref_attr = uc.file_2_seq(seq_f)

        refpath = us.RefSeq(''.join(ref_seqs), ref_name, jf.k)

        refpaths.append(refpath)

    for refpath in refpaths:

        finder = umf.MutationFinder(
            refpath, jf, args.steps, args.branchs, args.nodes
        )

        finder.graph_analysis()
        finder.quantify_paths(args.graphical)
        finder.quantify_clusters(args.graphical)

        for path in finder.get_paths(sort=True):
            sys.stdout.write(str(path) + "\n")

    sys.stdout.write("#Elapsed time:" + str(time.time() - time_start) + "\n")



import sys
import re

from .. utils import common as uc


def print_line(sample, region, location, type_var, removed,
               added, abnormal, normal, ratio, min_cov, min_exclu,
               variant, target, info, var_seq, ref_seq):
    line = "\t".join([sample, region, location, type_var, removed,
                      added, abnormal, normal, ratio, min_cov, min_exclu,
                      variant, target, info, var_seq, ref_seq])
    sys.stdout.write(line + "\n")


def print_vcf_header():
    header  = '##fileformat=VCFv4.1\n'
    header += '##INFO=<ID=TYPE,Number=A,Type=String,Description='
    header += '"The type of variant, either Insertion, ITD, I&I, Deletion, Substitution or Indel.">\n'
    header += '##INFO=<ID=TARGET,Number=A,Type=String,Description='
    header += '"Name of the sequencing that contains the mutation.">\n'
    header += '##INFO=<ID=RATIO,Number=A,Type=String,Description="Ratio of mutation to reference.">\n'
    header += '##INFO=<ID=MINCOV,Number=A,Type=String,Description='
    header += '"Minimum k-mer coverage of alternative allele.">\n'
    header += '##INFO=<ID=REMOVED,Number=A,Type=String,Description="Number of removed bases.">\n'
    header += '##INFO=<ID=ADDED,Number=A,Type=String,Description="Number of added bases.">\n'
    header += '#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\n'
    sys.stdout.write(header)


def print_vcf_line(chro, loc, ref_var, alt_var, type_var, target, ratio, min_cov, rem, ad):
    line = "\t".join([chro, str(loc), ".", ref_var, alt_var, ".", ".",
                      "TYPE="+type_var+";TARGET="+target+";RATIO="+ratio+";MINCOV="+min_cov +
                      ";REMOVED="+str(rem)+";ADDED="+str(ad)])
    sys.stdout.write(line + "\n")


def init_ref_seq(arg_ref):
    if not arg_ref:
        sys.exit("ERROR: Target file is empty\n")

    # BE 1-based
    nts = []
    chro = None
    strand = None

    # CODE for multiple >LOC lines:
    for line in open(arg_ref, "r"):
        line = line.strip()
        nt = []
        # Parse attributes
        if line[0] == '>':
            # sanity check
            loc = line.split(" ")[0]
            if ":" not in loc or "-" not in loc:
                sys.exit('ERROR: Fasta entries do not contain a correctly ' +
                         'formatted location: {}\n'.format(loc))

            # look up attributes in fasta file
            line = line.replace(">", "location=", 1)
            attr = {x.split("=")[0].strip(): x.split("=")[1].strip() for x in line.split("|")}
            exon = attr["location"]
            chro, pos = exon.split(":")
            refstart, refstop = pos.split("-")

            # get nt coordinates on the genome
            if 'strand' not in list(attr.keys()):
                attr['strand'] = '+'
                sys.stderr.write("WARNING: Strand is assumed to be '+' \n")
            strand = attr["strand"]
            for i in range(int(refstart), int(refstop) + 1):
                nt += [i]
            nt = nt[::-1] if strand == "-" else nt
            nts.extend(nt)

    return nts, chro, strand


def create_report(args):

    # Find correct extremities of a mutation
    sys.setrecursionlimit(10000)

    def get_extremities(va, p, rs):
        if p - 1 > 0 and rs[p - 1] == va[-1]:
            return get_extremities(rs[p - 1] + va[:-1], p - 1, rs)
        return p - 1

    if args.format == "vcf" and args.info == "cluster":
        # Note: could salvage that option if we get the fill ref from vs_ref entries
        sys.exit("ERROR: -f vcf and -i cluster options are incompatible")

    variants = {}
    samples = {}
    data = {}
    vcf = True if args.format == 'vcf' else False
    table = True if args.format == 'table' else False

    (nts, chro, strand) = init_ref_seq(args.target)

    if vcf:
        print_vcf_header()
    elif not table:
        print_line("Sample", "Region", "Location", "Type", "Removed",
                   "Added", "Abnormal", "Normal", "rVAF", "Min_coverage",
                   "Exclu_min_cov", "Variant", "Target", "Info", "Variant_sequence",
                   "Reference_sequence")

    for line in args.infile:
        # filter header
        if line[0] == "#":
            # sys.stderr.write("Filtred: " + line)
            continue

        tok = line.strip("\n").split("\t")

        # filter on info column
        if not re.search(args.info, line) or tok[0] == "Database" or len(tok) <= 1:
            # sys.stderr.write("Filtered: " + line)
            continue

        samp = tok[0]
        query = tok[1]
        ratio = tok[4]
        alt_exp = tok[5]
        ref_exp = tok[9]
        min_cov = tok[6]
        start_off = tok[7]
        alt_seq = tok[8]
        refSeq = tok[10]
        info = tok[11]

        min_exclu = ""
        variant = (tok[2], tok[3])
        ref_seq = refSeq.upper()

        if args.exclu != "" and alt_seq != "":
            res = uc.get_cov(args.exclu, alt_seq)
            min_exclu = str(res[2])

        if int(min_cov) < args.min_cov:
            continue

        # case: entries with no mutations
        if variant[0] == 'Reference':
            mod = ""
            if strand == "-":
                region = "{}:{}-{}".format(chro, nts[-1], nts[0])
            else:
                region = "{}:{}-{}".format(chro, nts[0], nts[-1])
            if not vcf and not table:
                print_line(samp, region, '-', variant[0], '0', '0',
                           '0.0', alt_exp, tok[4], min_cov, min_exclu, '-',
                           query, tok[-1], "", "")
                continue
            elif vcf:
                continue

        # case: there is a mutation
        else:
            start, mod, stop = variant[1].split(":")
            delet, insert = mod.split("/")

            added = str(len(insert))
            removed = str(len(delet))

            # start and end positions in 0-based coordinates
            pos = int(start) - 1
            pos -= int(start_off)
            end = int(stop) - 2  # one to go back to last position, the other for 0-base
            end -= int(start_off)

            if strand == "+":
                start_pos = nts[pos]
                end_pos = nts[end]
            elif strand == "-":
                start_pos = nts[end]
                end_pos = nts[pos]

            region = "{}:{}-{}".format(chro, start_pos, end_pos + 1)

            ref_var = delet.upper()
            alt_var = insert.upper()
            loc_var = start_pos
            end_var = end_pos

            if len(delet) == 0 and len(insert) != 0:
                if strand == "+":
                    start_pos = nts[pos]
                    end_pos = nts[end + 1]  # insertions end at last position
                elif strand == "-":
                    start_pos = nts[end + 1]
                    end_pos = nts[pos]
                region = "{}:{}-{}".format(chro, start_pos, end_pos + 1)

                var = insert.upper()
                ibef = get_extremities(var, pos, ref_seq)  # include current position
                before = ref_seq[ibef:pos]
                iaft = get_extremities(var[::-1], len(ref_seq)-pos, ref_seq[::-1])
                after = ref_seq[::-1][iaft:len(ref_seq)-pos][::-1]
                iaft = len(ref_seq) - iaft - 1
                ref_var = before + after
                alt_var = before + var + after
                loc_var = nts[iaft] if strand == "-" else nts[ibef]
                end_var = nts[iaft-len(ref_var)+1] if strand == "-" else nts[ibef+len(ref_var)-1]

                if loc_var + len(ref_var) - 1 != end_var and vcf:
                    sys.stderr.write("NOTE: Mutation overlaps 2 exons or more, VCF output is disabled \n")
                    continue

                # Reinterpret mutations for small ITDs
                # careful, going upstream may put us outside the reference.
                upstream = alt_seq[pos-len(insert):pos]
                match = 0
                if pos-len(insert) >= 0:
                    for i in range(0, len(insert)):
                        if insert[i] == upstream[i]:
                            match += 1
                    match = float(match)/len(insert)

                insert_type = "Insertion"
                if pos-len(insert) >= 0 and len(insert) >= 3 and insert == upstream:
                    insert_type = "ITD"
                    added += " | " + str(end_pos - start_pos + 1)
                elif pos-len(insert) >= 0 and len(insert) >= 3 and match > 0.5:
                    insert_type = "I&I"
                    added += " | " + str(end_pos - start_pos + 1)

                location = chro + ":" + str(end_pos)

            elif variant[0] == 'Deletion':
                region = "{}:{}-{}".format(chro, start_pos, end_pos + 1)
                location = ""
                insert_type = variant[0]

                var = delet.upper()
                ibef = get_extremities(var, pos, ref_seq)
                before = ref_seq[ibef:pos]
                iaft = get_extremities(var[::-1], len(ref_seq)-pos-1-len(var)+1, ref_seq[::-1])
                after = ref_seq[::-1][iaft:len(ref_seq)-pos-1-len(var)+1][::-1]
                iaft = len(ref_seq) - iaft - 1
                ref_var = before + var + after
                alt_var = before + after
                loc_var = nts[iaft] if strand == "-" else nts[ibef]
                end_var = nts[iaft-len(ref_var)+1] if strand == "-" else nts[ibef+len(ref_var)-1]

                if loc_var + len(ref_var) - 1 != end_var and vcf:
                    continue

            elif variant[0] == 'Substitution':
                location = chro + ":" + str(start_pos)
                insert_type = variant[0]

                if loc_var + len(ref_var) - 1 != end_var and vcf:
                    sys.stderr.write("NOTE: Mutation overlaps 2 exons or more, VCF output is disabled \n")
                    continue

            elif variant[0] == 'Indel':
                location = chro + ":" + str(end_pos)
                insert_type = variant[0]

                ref_var = ref_seq[pos-1] + delet.upper() + ref_seq[end + 1]
                alt_var = ref_seq[pos-1] + insert.upper() + ref_seq[end + 1]
                loc_var = start_pos - 1
                end_var = end_pos + 1

                if loc_var + len(ref_var) - 1 != end_var and vcf:
                    sys.stderr.write("NOTE: Mutation overlaps 2 exons or more, VCF output is disabled \n")
                    continue

            else:
                sys.stderr.write("WARNING: This variant isn't taken account\n")
                sys.stderr.write(" - variant: " + str(variant[0]) + "\n")
                sys.stderr.write(" - line: " + line)
                sys.exit()

        if not vcf and not table:
            print_line(samp, region, location, insert_type,
                       removed, added, alt_exp, ref_exp, ratio,
                       min_cov, min_exclu, mod, query, info,
                       alt_seq, refSeq)

        elif vcf:
            complement = str.maketrans('ATGCU', 'TACGA')
            ref_var = ref_var.translate(complement)[::-1] if strand == '-' else ref_var
            alt_var = alt_var.translate(complement)[::-1] if strand == '-' else alt_var
            print_vcf_line(chro, loc_var, ref_var, alt_var, insert_type,
                           query, ratio, min_cov, removed, added.replace(" ", ""))

        elif table:
            var_name = variant[0] + "/" + query if "/" not in variant[0] else variant[0]
            region_mod = region + ":" + mod if mod else region
            var = (var_name, region_mod)
            if var not in variants:
                variants[var] = 0
            variants[var] += 1

            if samp not in samples:
                samples[samp] = set()
            samples[samp].add(var)

            if samp not in data:
                data[samp] = {}
            data[samp][var] = float(ratio)

    if table:
        sorted_variants = sorted(variants, key=variants.get, reverse=True)

        sys.stdout.write("Sample")
        for v in sorted_variants:
            if v[0].split("/")[0] == "Reference":
                sys.stdout.write("\t" + v[0])
            else:
                sys.stdout.write("\t" + v[1])
        sys.stdout.write("\n")

        for s, sv in samples.items():
            sys.stdout.write(s)
            for v in sorted_variants:
                if v in sv:
                    if 'Reference' not in v[0] and (not data[s][v]):
                        sys.stdout.write("\t" + ".")
                    else:
                        sys.stdout.write("\t" + str(data[s][v]))
                else:
                    sys.stdout.write("\t" + ".")
            sys.stdout.write("\n")


def main_find_report(args, argparser):

    if args.infile.isatty() or args.target is None:
        argparser.print_help()
        sys.exit()

    create_report(args)



#                             -*- Mode: Python -*-
# Sequence.py
#

import sys

from . import common as uc


class RefSeq:
    """Reference path object that contains information on
    sequence, sequence name, kmers and kmer indices.

    Attributes
    ---------
    seq : str
        Reference sequence from fasta file.
    k : int
        K-mer size.
    first_kmer : str
        First kmer in seq.
    last_kmer : str
        Last kmer in seq.
    name : str
        Path name.
    ref_mer : list
        List of kmers found in seq.

    Methods
    -------
    set_index
    """

    def __init__(self, seq, name, k):

        self.seq = seq
        self.k = k

        self.first_kmer = self.seq[0:self.k]
        self.last_kmer = self.seq[-self.k:]

        self.name = name

        self.ref_mer = uc.get_ref_kmer(seq, self.name, self.k)

        assert len(self.ref_mer)

    def set_index(self, kmer):
        """"""

        self.seq_index = tuple([kmer.index(k) for k in self.ref_mer])
        self.first_ix = self.seq_index[0]
        self.last_ix = self.seq_index[-1]

    def __getitem__(self, item):
        try:
            return self.seq_index[item]
        except AttributeError:
            sys.stderr.write('Attribute `seq_index` is not set yet\n')
            return None


class AltSeq:
    """"""

    def __init__(self, alt_index, finder):
        """"""

        self.finder = finder

        self.seq_index = alt_index
        self.seq = self.finder.get_seq(self.seq_index, skip_prefix=False)
        self.first_ix = self.seq_index[0]
        self.last_ix = self.seq_index[-1]

        self.seq_len = len(self.seq_index)

        self.ref_index = self.finder.refpath.seq_index
        self.ref_name = self.finder.refpath.name

    def __getitem__(self, item):
        return self.seq_index[item]
